@@ -147,7 +147,7 @@ class MicroPythonBackend(MainBackend, ABC):
         except Exception:
             logger.exception("Crash in backend")
 
-    def _prepare_after_soft_reboot(self, clean):
+    def _prepare_after_soft_reboot(self, clean=False):
         self._report_time("bef preparing helpers")
         script = self._get_all_helpers()
         self._check_perform_just_in_case_gc()
@@ -205,20 +205,20 @@ class MicroPythonBackend(MainBackend, ABC):
                 except ImportError:
                     import os
                 import sys
+                import builtins
                 
                 # for object inspector
                 inspector_values = dict()
-                last_repl_values = []
                 @classmethod
                 def print_repl_value(cls, obj):
+                    global _
                     if obj is not None:
-                        cls.last_repl_values.append(obj)
-                        cls.last_repl_values = cls.last_repl_values[-{num_values_to_keep}:]
-                        print({start_marker!r} % id(obj), repr(obj), {end_marker!r}, sep='')
+                        cls.builtins.print({start_marker!r} % id(obj), repr(obj), {end_marker!r}, sep='')
+                        _ = obj
                 
-                @staticmethod
-                def print_mgmt_value(obj):
-                    print({mgmt_start!r}, repr(obj), {mgmt_end!r}, sep='', end='')
+                @classmethod
+                def print_mgmt_value(cls, obj):
+                    cls.builtins.print({mgmt_start!r}, repr(obj), {mgmt_end!r}, sep='', end='')
                     
                 @staticmethod
                 def repr(obj):
@@ -238,7 +238,6 @@ class MicroPythonBackend(MainBackend, ABC):
                         return [rec[0] for rec in cls.os.ilistdir(x) if rec[0] not in ('.', '..')]
             """
             ).format(
-                num_values_to_keep=self._get_num_values_to_keep(),
                 start_marker=OBJECT_LINK_START,
                 end_marker=OBJECT_LINK_END,
                 mgmt_start=MGMT_VALUE_START.decode(ENCODING),
@@ -251,11 +250,6 @@ class MicroPythonBackend(MainBackend, ABC):
     def _get_custom_helpers(self):
         return ""
 
-    def _get_num_values_to_keep(self):
-        """How many last evaluated REPL values and visited Object inspector values to keep
-        in internal lists for the purpose of retrieving them by id for Object inspector"""
-        return 5
-
     def _sync_time(self):
         raise NotImplementedError()
 
@@ -351,16 +345,34 @@ class MicroPythonBackend(MainBackend, ABC):
             except ConnectionClosedException as e:
                 self._on_connection_closed(e)
             except ManagementError as e:
+                logger.info("Caught ManagementError", exc_info=e)
                 if "KeyboardInterrupt" in e.err:
                     response = create_error_response(error="Interrupted", interrupted=True)
                 else:
-                    self._send_output("THONNY FAILED TO EXECUTE COMMAND %s\n" % cmd.name, "stderr")
+                    self._show_error("\nNB! Thonny could not execute '%s'." % cmd.name)
                     # traceback.print_exc() # I'll know the trace from command
-                    self._show_error("\n")
-                    self._show_error("SCRIPT:\n" + e.script + "\n")
-                    self._show_error("STDOUT:\n" + e.out + "\n")
-                    self._show_error("STDERR:\n" + e.err + "\n")
+                    if self._management_error_is_not_thonnys_fault(e):
+                        self._show_error(
+                            (
+                                "It looks like %s has arrived to an unexpected state"
+                                " or there have been communication problems."
+                                " See Thonny's backend.log for more information."
+                            )
+                            % (
+                                "CircuitPython"
+                                if self._connected_to_circuitpython()
+                                else "MicroPython"
+                            )
+                        )
+                    else:
+                        self._show_error("\n")
+                        self._show_error("SCRIPT:\n" + e.script + "\n")
+                        self._show_error("STDOUT:\n" + e.out + "\n")
+                        self._show_error("STDERR:\n" + e.err + "\n")
 
+                    self._show_error(
+                        "You may need to reconnect, Stop/Restart or hard-reset your device.\n"
+                    )
                     response = create_error_response(error="ManagementError")
             except Exception as e:
                 _report_internal_error(e)
@@ -574,6 +586,21 @@ class MicroPythonBackend(MainBackend, ABC):
         "Invoked between commands"
         raise NotImplementedError()
 
+    def _management_error_is_not_thonnys_fault(self, e):
+        outerr = ""
+        if e.out:
+            outerr += e.out
+        if e.err:
+            outerr += e.err
+        # https://github.com/micropython/micropython/issues/7171
+        # https://github.com/micropython/micropython/issues/6899
+        return (
+            "NameError: name '__thonny_helper' isn't defined" in outerr
+            or "SyntaxError" in outerr
+            or "IndentationError" in outerr
+            or "UnicodeDecodeError" in outerr
+        )
+
     def _check_for_side_commands(self):
         # NB! EOFCommand gets different treatment depending whether it is read during processing a command
         # (ie. here) or it gets read when REPL is idle (ie. in mainloop)
@@ -655,6 +682,7 @@ class MicroPythonBackend(MainBackend, ABC):
         for name, pair in globs.items():
             value_infos[name] = ValueInfo(pair[1], pair[0])
 
+        logger.debug("Returning %d globals", len(value_infos))
         return {"module_name": cmd.module_name, "globals": value_infos}
 
     def _cmd_get_fs_info(self, cmd):
@@ -706,7 +734,6 @@ class MicroPythonBackend(MainBackend, ABC):
                 """
                 for __thonny_helper.object_info in (
                         list(globals().values()) 
-                        + __thonny_helper.last_repl_values
                         + list(__thonny_helper.inspector_values.values())):
                     if id(__thonny_helper.object_info) == %d:
                         __thonny_helper.print_mgmt_value({
@@ -752,6 +779,7 @@ class MicroPythonBackend(MainBackend, ABC):
                             break
                     else:
                         __thonny_helper.object_info = None
+                        __thonny_helper.print_mgmt_value(None)
                         
                 __thonny_helper.context_value = None
                 __thonny_helper.context_children = None
@@ -763,7 +791,7 @@ class MicroPythonBackend(MainBackend, ABC):
             return None
 
     def _get_object_attributes(self, all_attributes):
-        """object is given in __thonny_helper.object_info """
+        """object is given in __thonny_helper.object_info"""
         atts = self._evaluate(
             "{name : ("
             "   id(getattr(__thonny_helper.object_info, name)),"
@@ -777,7 +805,7 @@ class MicroPythonBackend(MainBackend, ABC):
         }
 
     def _get_object_info_extras(self, type_name):
-        """object is given in __thonny_helper.object_info """
+        """object is given in __thonny_helper.object_info"""
         if type_name in ("list", "tuple", "set"):
             items = self._evaluate(
                 "[(id(x), __thonny_helper.repr(x)) for x in __thonny_helper.object_info]"
