@@ -469,8 +469,8 @@ class BareMetalMicroPythonBackend(MicroPythonBackend, UploadDownloadMixin):
                 % ((out + err).encode(ENCODING) + self._last_prompt)
             )
 
-    def _ensure_normal_mode(self):
-        if self._last_prompt == NORMAL_PROMPT:
+    def _ensure_normal_mode(self, force=False):
+        if self._last_prompt == NORMAL_PROMPT and not force:
             return
 
         logger.debug("requesting normal mode at %r", self._last_prompt)
@@ -484,10 +484,12 @@ class BareMetalMicroPythonBackend(MicroPythonBackend, UploadDownloadMixin):
         logger.debug("_soft_reboot_without_running_main")
         self._interrupt_to_raw_prompt()
         self._soft_reboot_in_raw_prompt_without_running_main()
+        logger.debug("Done _soft_reboot_without_running_main")
 
     def _soft_reboot_for_restarting_user_program(self):
         # Need to go to normal mode. MP doesn't run user code in raw mode
         # (CP does, but it doesn't hurt to do it there as well)
+        logger.debug("_soft_reboot_for_restarting_user_program")
         self._ensure_normal_mode()
         self._write(SOFT_REBOOT_CMD)
         self._check_reconnect()
@@ -495,10 +497,12 @@ class BareMetalMicroPythonBackend(MicroPythonBackend, UploadDownloadMixin):
         logger.debug("Restoring helpers")
         self._prepare_after_soft_reboot(False)
         self.send_message(ToplevelResponse(cwd=self._cwd))
+        logger.debug("_soft_reboot_for_restarting_user_program")
 
     def _check_reconnect(self):
         if self._connected_over_webrepl():
             time.sleep(1)
+            logger.info("Reconnecting to WebREPL")
             self._connection = self._connection.close_and_return_new_connection()
 
     def _connected_over_webrepl(self):
@@ -531,7 +535,7 @@ class BareMetalMicroPythonBackend(MicroPythonBackend, UploadDownloadMixin):
         with self._interrupt_lock:
             while to_be_written:
                 block = self._extract_block_without_splitting_chars(to_be_written)
-                self._write(to_be_written)
+                self._write(block)
                 # Try to consume the echo
                 echo += self._connection.soft_read(len(block), timeout=1)
                 to_be_written = to_be_written[len(block) :]
@@ -558,7 +562,7 @@ class BareMetalMicroPythonBackend(MicroPythonBackend, UploadDownloadMixin):
         self._forward_unexpected_output()
 
         to_be_sent = script.encode("UTF-8")
-        logger.debug("Submitting via %s: %r", self._submit_mode, to_be_sent[:50])
+        logger.debug("Submitting via %s: %r", self._submit_mode, to_be_sent[:70])
         with self._interrupt_lock:
             if self._submit_mode == PASTE_SUBMIT_MODE:
                 self._submit_code_via_paste_mode(to_be_sent)
@@ -950,15 +954,22 @@ class BareMetalMicroPythonBackend(MicroPythonBackend, UploadDownloadMixin):
 
     def _cmd_Run(self, cmd):
         """Only for %run $EDITOR_CONTENT. start runs will be handled differently."""
+        reset_environment_before_run = True  # TODO: make it configurable
         if cmd.get("source"):
-            self._soft_reboot_without_running_main()
+            if reset_environment_before_run:
+                self._soft_reboot_without_running_main()
+
             if self._submit_mode == PASTE_SUBMIT_MODE:
                 source = self._avoid_printing_expression_statements(cmd.source)
+                if reset_environment_before_run:
+                    logger.debug("Ensuring normal mode after soft reboot")
+                    self._ensure_normal_mode(force=True)
             else:
                 source = cmd.source
 
             self._execute(source, capture_output=False)
-            self._prepare_after_soft_reboot(False)
+            if reset_environment_before_run:
+                self._prepare_after_soft_reboot(False)
         return {}
 
     def _cmd_execute_system_command(self, cmd):
@@ -1241,7 +1252,11 @@ class BareMetalMicroPythonBackend(MicroPythonBackend, UploadDownloadMixin):
             capture_output=True,
         )
 
-        if "readonly" in (out + err).replace("-", "").lower():
+        canonic_out = (out + err).replace("-", "").lower()
+        if "readonly" in canonic_out or "errno 30" in canonic_out:
+            logger.info(
+                "Could not write via serial, got '%s'. Assuming read-only filesystem.", out + err
+            )
             raise ReadOnlyFilesystemError()
         elif out + err:
             raise RuntimeError(
