@@ -4,8 +4,7 @@ import subprocess
 import tempfile
 import threading
 import tkinter as tk
-from tkinter.messagebox import showerror
-from tkinter import ttk
+from tkinter import ttk, messagebox
 from typing import cast, Tuple
 
 from thonny import running, get_runner, get_workbench, ui_utils
@@ -13,7 +12,6 @@ from thonny.common import InlineCommand
 from thonny.languages import tr
 from thonny.plugins.files import upload, prepare_upload_items
 from thonny.plugins.micropython import MicroPythonProxy, LocalMicroPythonProxy
-from thonny.plugins.micropython.micropip import MICROPYTHON_ORG_JSON
 from thonny.plugins.pip_gui import (
     BackendPipDialog,
     _fetch_url_future,
@@ -22,6 +20,8 @@ from thonny.plugins.pip_gui import (
 from thonny.running import InlineCommandDialog
 from thonny.workdlg import SubprocessDialog
 
+MICROPYTHON_ORG_JSON = "https://micropython.org/pi/%s/json"
+
 
 class MicroPythonPipDialog(BackendPipDialog):
     def __init__(self, master):
@@ -31,7 +31,7 @@ class MicroPythonPipDialog(BackendPipDialog):
         assert isinstance(self._backend_proxy, MicroPythonProxy)
 
     def _create_pip_process(self, args):
-        return self._create_python_process(["-m", "thonny.plugins.micropython.micropip"] + args)
+        return self._create_python_process(["-m", "thonny.plugins.micropython.minipip"] + args)
 
     def _get_active_version(self, name):
         # Don't have dist-level information
@@ -39,6 +39,16 @@ class MicroPythonPipDialog(BackendPipDialog):
 
     def _on_install_click(self):
         if self.install_button["text"] == self.get_install_button_text():
+            if not self._looks_like_micropython_package():
+                if not messagebox.askyesno(
+                    title=tr("Confirmation"),
+                    message=tr(
+                        "This doesn't look like MicroPython/CircuitPython package.\n"
+                        "Are you sure you want to install it?"
+                    ),
+                    parent=self,
+                ):
+                    return
             super()._on_install_click()
         elif self.install_button["text"] == self.get_search_button_text():
             self.search_box.delete(0, "end")
@@ -51,6 +61,24 @@ class MicroPythonPipDialog(BackendPipDialog):
                 "Unexpected text '%s' on install button" % self.install_button["text"]
             )
 
+    def _looks_like_micropython_package(self):
+        assert self.current_package_data is not None
+        name = self.current_package_data["info"]["name"]
+        classifiers = self.current_package_data["info"].get("classifiers", [])
+        print(classifiers)
+        for token in ["micropython", "circuitpython", "pycopy"]:
+            if token in name.lower():
+                return True
+
+        for mp_class in [
+            "Programming Language :: Python :: Implementation :: MicroPython",
+            "Programming Language :: Python :: Implementation :: CircuitPython",
+        ]:
+            if mp_class in classifiers:
+                return True
+
+        return False
+
     def _on_uninstall_click(self):
         if self.uninstall_button["text"] == self.get_uninstall_button_text():
             super()._on_uninstall_click()
@@ -78,7 +106,7 @@ class MicroPythonPipDialog(BackendPipDialog):
         )
 
     def _get_install_command(self):
-        return ["install", "-p", self._current_temp_dir]
+        return ["install", "--target", self._current_temp_dir]
 
     def _perform_pip_action(self, action: str) -> bool:
         if self._perform_pip_action_without_refresh(action):
@@ -330,14 +358,14 @@ class MicroPythonPipDialog(BackendPipDialog):
         return []
 
     def _run_pip_with_dialog(self, args, title) -> Tuple[int, str, str]:
-        args = ["-m", "thonny.plugins.micropython.micropip"] + args
+        args = ["-m", "thonny.plugins.micropython.minipip"] + args
         proc = running.create_frontend_python_process(args, stderr=subprocess.STDOUT)
         cmd = proc.cmd
         dlg = InstallAndUploadDialog(
             self,
             proc,
             back_cmd=self._create_upload_command,
-            title="micropip",
+            title="minipip",
             instructions=title,
             autostart=True,
             output_prelude=subprocess.list2cmdline(cmd) + "\n",
@@ -349,10 +377,10 @@ class MicroPythonPipDialog(BackendPipDialog):
 
 class LocalMicroPythonPipDialog(MicroPythonPipDialog):
     def _get_install_command(self):
-        return ["install", "-p", self._get_target_directory()]
+        return ["install", "--target", self._get_target_directory()]
 
     def _upload_installed_files(self) -> bool:
-        "nothing to do -- micropip installed files directly to the right directory"
+        "nothing to do -- minipip installed files directly to the right directory"
 
     def _delete_paths(self, paths):
         # assuming all files are listed if their directory is listed
@@ -363,10 +391,10 @@ class LocalMicroPythonPipDialog(MicroPythonPipDialog):
                 os.removedirs(path)
 
     def _run_pip_with_dialog(self, args, title) -> Tuple[int, str, str]:
-        args = ["-m", "thonny.plugins.micropython.micropip"] + args
+        args = ["-m", "thonny.plugins.micropython.minipip"] + args
         proc = running.create_frontend_python_process(args, stderr=subprocess.STDOUT)
         cmd = proc.cmd
-        dlg = SubprocessDialog(self, proc, "micropip", long_description=title, autostart=True)
+        dlg = SubprocessDialog(self, proc, "minipip", long_description=title, autostart=True)
         ui_utils.show_dialog(dlg)
         return dlg.returncode, dlg.stdout, dlg.stderr
 
@@ -428,7 +456,7 @@ class InstallAndUploadDialog(InlineCommandDialog):
         self.returncode = self._proc.wait()
         if self.returncode:
             self.set_action_text("Error")
-            self.append_text("\nmicropip returned with error code %s\n" % self.returncode)
+            self.append_text("\nminipip returned with error code %s\n" % self.returncode)
         else:
             self.set_action_text("Copying to the device")
             self.append_text("Copying to the device\n")
